import httpx

from ...common.internal import get_forced_url, get_global_unique_hash
from ...common.settings import settings
from ..types import ResponseError, SandboxConfiguration


class SyncSandboxAction:
    def __init__(self, sandbox_config: SandboxConfiguration):
        self.sandbox_config = sandbox_config

    @property
    def name(self) -> str:
        return self.sandbox_config.metadata.name if self.sandbox_config.metadata else ""

    @property
    def external_url(self) -> str:
        if (
            self.sandbox_config.metadata
            and self.sandbox_config.metadata.url is not None
            and self.sandbox_config.metadata.url != ""
        ):
            return self.sandbox_config.metadata.url

        return f"{settings.run_url}/{settings.workspace}/sandboxes/{self.name}"

    @property
    def internal_url(self) -> str:
        hash_value = get_global_unique_hash(settings.workspace, "sandbox", self.name)
        return f"{settings.run_internal_protocol}://bl-{settings.env}-{hash_value}.{settings.run_internal_hostname}"

    @property
    def forced_url(self) -> str | None:
        if self.sandbox_config.force_url:
            return self.sandbox_config.force_url
        return get_forced_url("sandbox", self.name)

    @property
    def url(self) -> str:
        if self.forced_url:
            url = self.forced_url
            return url[:-1] if url.endswith("/") else url
        return self.external_url

    @property
    def fallback_url(self) -> str | None:
        if self.external_url != self.url:
            return self.external_url
        return None

    def get_client(self) -> httpx.Client:
        if self.sandbox_config.force_url:
            return httpx.Client(
                base_url=self.sandbox_config.force_url, headers=self.sandbox_config.headers
            )
        return httpx.Client(
            base_url=self.url,
            headers={**settings.headers, **self.sandbox_config.headers},
        )

    def handle_response_error(self, response: httpx.Response):
        if not response.is_success:
            raise ResponseError(response)





"""Contains all the data models used in inputs/outputs"""

from .acl import ACL
from .agent import Agent
from .agent_spec import AgentSpec
from .api_key import ApiKey
from .billable_time_metric import BillableTimeMetric
from .check_workspace_availability_body import CheckWorkspaceAvailabilityBody
from .configuration import Configuration
from .continent import Continent
from .core_event import CoreEvent
from .core_spec import CoreSpec
from .core_spec_configurations import CoreSpecConfigurations
from .country import Country
from .create_api_key_for_service_account_body import CreateApiKeyForServiceAccountBody
from .create_job_execution_request import CreateJobExecutionRequest
from .create_job_execution_request_tasks_item import CreateJobExecutionRequestTasksItem
from .create_job_execution_response import CreateJobExecutionResponse
from .create_job_execution_response_tasks_item import CreateJobExecutionResponseTasksItem
from .create_workspace_service_account_body import CreateWorkspaceServiceAccountBody
from .create_workspace_service_account_response_200 import CreateWorkspaceServiceAccountResponse200
from .custom_domain import CustomDomain
from .custom_domain_metadata import CustomDomainMetadata
from .custom_domain_spec import CustomDomainSpec
from .custom_domain_spec_txt_records import CustomDomainSpecTxtRecords
from .delete_sandbox_preview_token_response_200 import DeleteSandboxPreviewTokenResponse200
from .delete_volume_template_version_response_200 import DeleteVolumeTemplateVersionResponse200
from .delete_workspace_service_account_response_200 import DeleteWorkspaceServiceAccountResponse200
from .entrypoint import Entrypoint
from .entrypoint_env import EntrypointEnv
from .expiration_policy import ExpirationPolicy
from .flavor import Flavor
from .form import Form
from .form_config import FormConfig
from .form_oauth import FormOauth
from .form_secrets import FormSecrets
from .function import Function
from .function_spec import FunctionSpec
from .get_workspace_service_accounts_response_200_item import (
    GetWorkspaceServiceAccountsResponse200Item,
)
from .histogram_bucket import HistogramBucket
from .histogram_stats import HistogramStats
from .image import Image
from .image_metadata import ImageMetadata
from .image_spec import ImageSpec
from .image_tag import ImageTag
from .integration import Integration
from .integration_additional_infos import IntegrationAdditionalInfos
from .integration_connection import IntegrationConnection
from .integration_connection_spec import IntegrationConnectionSpec
from .integration_connection_spec_config import IntegrationConnectionSpecConfig
from .integration_connection_spec_secret import IntegrationConnectionSpecSecret
from .integration_endpoint import IntegrationEndpoint
from .integration_endpoint_token import IntegrationEndpointToken
from .integration_endpoints import IntegrationEndpoints
from .integration_headers import IntegrationHeaders
from .integration_model import IntegrationModel
from .integration_organization import IntegrationOrganization
from .integration_query_params import IntegrationQueryParams
from .integration_repository import IntegrationRepository
from .invite_workspace_user_body import InviteWorkspaceUserBody
from .job import Job
from .job_execution import JobExecution
from .job_execution_config import JobExecutionConfig
from .job_execution_metadata import JobExecutionMetadata
from .job_execution_spec import JobExecutionSpec
from .job_execution_stats import JobExecutionStats
from .job_execution_task import JobExecutionTask
from .job_execution_task_condition import JobExecutionTaskCondition
from .job_execution_task_metadata import JobExecutionTaskMetadata
from .job_execution_task_spec import JobExecutionTaskSpec
from .job_metrics import JobMetrics
from .job_metrics_executions_total import JobMetricsExecutionsTotal
from .job_metrics_tasks_total import JobMetricsTasksTotal
from .job_spec import JobSpec
from .jobs_chart_value import JobsChartValue
from .jobs_network_chart import JobsNetworkChart
from .jobs_success_failed_chart import JobsSuccessFailedChart
from .jobs_total import JobsTotal
from .last_n_requests_metric import LastNRequestsMetric
from .latency_metric import LatencyMetric
from .location_response import LocationResponse
from .logs_response import LogsResponse
from .logs_response_data import LogsResponseData
from .mcp_definition import MCPDefinition
from .mcp_definition_entrypoint import MCPDefinitionEntrypoint
from .mcp_definition_form import MCPDefinitionForm
from .memory_allocation_by_name import MemoryAllocationByName
from .memory_allocation_metric import MemoryAllocationMetric
from .metadata import Metadata
from .metadata_labels import MetadataLabels
from .metric import Metric
from .metrics import Metrics
from .metrics_models import MetricsModels
from .metrics_request_total_per_code import MetricsRequestTotalPerCode
from .metrics_rps_per_code import MetricsRpsPerCode
from .model import Model
from .model_private_cluster import ModelPrivateCluster
from .model_spec import ModelSpec
from .o_auth import OAuth
from .owner_fields import OwnerFields
from .pending_invitation import PendingInvitation
from .pending_invitation_accept import PendingInvitationAccept
from .pending_invitation_render import PendingInvitationRender
from .pending_invitation_render_invited_by import PendingInvitationRenderInvitedBy
from .pending_invitation_render_workspace import PendingInvitationRenderWorkspace
from .pending_invitation_workspace_details import PendingInvitationWorkspaceDetails
from .pod_template_spec import PodTemplateSpec
from .policy import Policy
from .policy_location import PolicyLocation
from .policy_max_tokens import PolicyMaxTokens
from .policy_spec import PolicySpec
from .port import Port
from .preview import Preview
from .preview_metadata import PreviewMetadata
from .preview_spec import PreviewSpec
from .preview_spec_request_headers import PreviewSpecRequestHeaders
from .preview_spec_response_headers import PreviewSpecResponseHeaders
from .preview_token import PreviewToken
from .preview_token_metadata import PreviewTokenMetadata
from .preview_token_spec import PreviewTokenSpec
from .private_cluster import PrivateCluster
from .private_location import PrivateLocation
from .public_ip import PublicIp
from .public_ips import PublicIps
from .region import Region
from .repository import Repository
from .request_duration_over_time_metric import RequestDurationOverTimeMetric
from .request_duration_over_time_metrics import RequestDurationOverTimeMetrics
from .request_total_by_origin_metric import RequestTotalByOriginMetric
from .request_total_by_origin_metric_request_total_by_origin import (
    RequestTotalByOriginMetricRequestTotalByOrigin,
)
from .request_total_by_origin_metric_request_total_by_origin_and_code import (
    RequestTotalByOriginMetricRequestTotalByOriginAndCode,
)
from .request_total_metric import RequestTotalMetric
from .request_total_metric_request_total_per_code import RequestTotalMetricRequestTotalPerCode
from .request_total_metric_rps_per_code import RequestTotalMetricRpsPerCode
from .request_total_response_data import RequestTotalResponseData
from .resource import Resource
from .resource_log import ResourceLog
from .resource_log_chart import ResourceLogChart
from .resource_log_response import ResourceLogResponse
from .resource_metrics import ResourceMetrics
from .resource_metrics_request_total_per_code import ResourceMetricsRequestTotalPerCode
from .resource_metrics_request_total_per_code_previous import (
    ResourceMetricsRequestTotalPerCodePrevious,
)
from .resource_metrics_rps_per_code import ResourceMetricsRpsPerCode
from .resource_metrics_rps_per_code_previous import ResourceMetricsRpsPerCodePrevious
from .resource_trace import ResourceTrace
from .revision_configuration import RevisionConfiguration
from .revision_metadata import RevisionMetadata
from .runtime import Runtime
from .runtime_configuration import RuntimeConfiguration
from .runtime_startup_probe import RuntimeStartupProbe
from .sandbox import Sandbox
from .sandbox_definition import SandboxDefinition
from .sandbox_lifecycle import SandboxLifecycle
from .sandbox_metrics import SandboxMetrics
from .sandbox_spec import SandboxSpec
from .serverless_config import ServerlessConfig
from .serverless_config_configuration import ServerlessConfigConfiguration
from .spec_configuration import SpecConfiguration
from .start_sandbox import StartSandbox
from .stop_sandbox import StopSandbox
from .store_agent import StoreAgent
from .store_agent_labels import StoreAgentLabels
from .store_configuration import StoreConfiguration
from .store_configuration_option import StoreConfigurationOption
from .template import Template
from .template_variable import TemplateVariable
from .time_fields import TimeFields
from .time_to_first_token_over_time_metrics import TimeToFirstTokenOverTimeMetrics
from .token_rate_metric import TokenRateMetric
from .token_rate_metrics import TokenRateMetrics
from .token_total_metric import TokenTotalMetric
from .trace_ids_response import TraceIdsResponse
from .trigger import Trigger
from .trigger_configuration import TriggerConfiguration
from .trigger_configuration_task import TriggerConfigurationTask
from .update_workspace_service_account_body import UpdateWorkspaceServiceAccountBody
from .update_workspace_service_account_response_200 import UpdateWorkspaceServiceAccountResponse200
from .update_workspace_user_role_body import UpdateWorkspaceUserRoleBody
from .volume import Volume
from .volume_attachment import VolumeAttachment
from .volume_spec import VolumeSpec
from .volume_state import VolumeState
from .volume_template import VolumeTemplate
from .volume_template_spec import VolumeTemplateSpec
from .volume_template_state import VolumeTemplateState
from .volume_template_version import VolumeTemplateVersion
from .websocket_channel import WebsocketChannel
from .websocket_message import WebsocketMessage
from .workspace import Workspace
from .workspace_labels import WorkspaceLabels
from .workspace_runtime import WorkspaceRuntime
from .workspace_user import WorkspaceUser

__all__ = (
    "ACL",
    "Agent",
    "AgentSpec",
    "ApiKey",
    "BillableTimeMetric",
    "CheckWorkspaceAvailabilityBody",
    "Configuration",
    "Continent",
    "CoreEvent",
    "CoreSpec",
    "CoreSpecConfigurations",
    "Country",
    "CreateApiKeyForServiceAccountBody",
    "CreateJobExecutionRequest",
    "CreateJobExecutionRequestTasksItem",
    "CreateJobExecutionResponse",
    "CreateJobExecutionResponseTasksItem",
    "CreateWorkspaceServiceAccountBody",
    "CreateWorkspaceServiceAccountResponse200",
    "CustomDomain",
    "CustomDomainMetadata",
    "CustomDomainSpec",
    "CustomDomainSpecTxtRecords",
    "DeleteSandboxPreviewTokenResponse200",
    "DeleteVolumeTemplateVersionResponse200",
    "DeleteWorkspaceServiceAccountResponse200",
    "Entrypoint",
    "EntrypointEnv",
    "ExpirationPolicy",
    "Flavor",
    "Form",
    "FormConfig",
    "FormOauth",
    "FormSecrets",
    "Function",
    "FunctionSpec",
    "GetWorkspaceServiceAccountsResponse200Item",
    "HistogramBucket",
    "HistogramStats",
    "Image",
    "ImageMetadata",
    "ImageSpec",
    "ImageTag",
    "Integration",
    "IntegrationAdditionalInfos",
    "IntegrationConnection",
    "IntegrationConnectionSpec",
    "IntegrationConnectionSpecConfig",
    "IntegrationConnectionSpecSecret",
    "IntegrationEndpoint",
    "IntegrationEndpoints",
    "IntegrationEndpointToken",
    "IntegrationHeaders",
    "IntegrationModel",
    "IntegrationOrganization",
    "IntegrationQueryParams",
    "IntegrationRepository",
    "InviteWorkspaceUserBody",
    "Job",
    "JobExecution",
    "JobExecutionConfig",
    "JobExecutionMetadata",
    "JobExecutionSpec",
    "JobExecutionStats",
    "JobExecutionTask",
    "JobExecutionTaskCondition",
    "JobExecutionTaskMetadata",
    "JobExecutionTaskSpec",
    "JobMetrics",
    "JobMetricsExecutionsTotal",
    "JobMetricsTasksTotal",
    "JobsChartValue",
    "JobsNetworkChart",
    "JobSpec",
    "JobsSuccessFailedChart",
    "JobsTotal",
    "LastNRequestsMetric",
    "LatencyMetric",
    "LocationResponse",
    "LogsResponse",
    "LogsResponseData",
    "MCPDefinition",
    "MCPDefinitionEntrypoint",
    "MCPDefinitionForm",
    "MemoryAllocationByName",
    "MemoryAllocationMetric",
    "Metadata",
    "MetadataLabels",
    "Metric",
    "Metrics",
    "MetricsModels",
    "MetricsRequestTotalPerCode",
    "MetricsRpsPerCode",
    "Model",
    "ModelPrivateCluster",
    "ModelSpec",
    "OAuth",
    "OwnerFields",
    "PendingInvitation",
    "PendingInvitationAccept",
    "PendingInvitationRender",
    "PendingInvitationRenderInvitedBy",
    "PendingInvitationRenderWorkspace",
    "PendingInvitationWorkspaceDetails",
    "PodTemplateSpec",
    "Policy",
    "PolicyLocation",
    "PolicyMaxTokens",
    "PolicySpec",
    "Port",
    "Preview",
    "PreviewMetadata",
    "PreviewSpec",
    "PreviewSpecRequestHeaders",
    "PreviewSpecResponseHeaders",
    "PreviewToken",
    "PreviewTokenMetadata",
    "PreviewTokenSpec",
    "PrivateCluster",
    "PrivateLocation",
    "PublicIp",
    "PublicIps",
    "Region",
    "Repository",
    "RequestDurationOverTimeMetric",
    "RequestDurationOverTimeMetrics",
    "RequestTotalByOriginMetric",
    "RequestTotalByOriginMetricRequestTotalByOrigin",
    "RequestTotalByOriginMetricRequestTotalByOriginAndCode",
    "RequestTotalMetric",
    "RequestTotalMetricRequestTotalPerCode",
    "RequestTotalMetricRpsPerCode",
    "RequestTotalResponseData",
    "Resource",
    "ResourceLog",
    "ResourceLogChart",
    "ResourceLogResponse",
    "ResourceMetrics",
    "ResourceMetricsRequestTotalPerCode",
    "ResourceMetricsRequestTotalPerCodePrevious",
    "ResourceMetricsRpsPerCode",
    "ResourceMetricsRpsPerCodePrevious",
    "ResourceTrace",
    "RevisionConfiguration",
    "RevisionMetadata",
    "Runtime",
    "RuntimeConfiguration",
    "RuntimeStartupProbe",
    "Sandbox",
    "SandboxDefinition",
    "SandboxLifecycle",
    "SandboxMetrics",
    "SandboxSpec",
    "ServerlessConfig",
    "ServerlessConfigConfiguration",
    "SpecConfiguration",
    "StartSandbox",
    "StopSandbox",
    "StoreAgent",
    "StoreAgentLabels",
    "StoreConfiguration",
    "StoreConfigurationOption",
    "Template",
    "TemplateVariable",
    "TimeFields",
    "TimeToFirstTokenOverTimeMetrics",
    "TokenRateMetric",
    "TokenRateMetrics",
    "TokenTotalMetric",
    "TraceIdsResponse",
    "Trigger",
    "TriggerConfiguration",
    "TriggerConfigurationTask",
    "UpdateWorkspaceServiceAccountBody",
    "UpdateWorkspaceServiceAccountResponse200",
    "UpdateWorkspaceUserRoleBody",
    "Volume",
    "VolumeAttachment",
    "VolumeSpec",
    "VolumeState",
    "VolumeTemplate",
    "VolumeTemplateSpec",
    "VolumeTemplateState",
    "VolumeTemplateVersion",
    "WebsocketChannel",
    "WebsocketMessage",
    "Workspace",
    "WorkspaceLabels",
    "WorkspaceRuntime",
    "WorkspaceUser",
)



from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.custom_domain_metadata import CustomDomainMetadata
    from ..models.custom_domain_spec import CustomDomainSpec


T = TypeVar("T", bound="CustomDomain")


@_attrs_define
class CustomDomain:
    """Custom domain for preview deployments
    The custom domain represents a base domain (e.g., example.com) that will be used
    to serve preview deployments. Each preview will be accessible at a subdomain:
    <preview-id>.preview.<base-domain> (e.g., abc123.preview.example.com)

        Attributes:
            metadata (Union[Unset, CustomDomainMetadata]): Custom domain metadata
            spec (Union[Unset, CustomDomainSpec]): Custom domain specification
    """

    metadata: Union[Unset, "CustomDomainMetadata"] = UNSET
    spec: Union[Unset, "CustomDomainSpec"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        metadata: Union[Unset, dict[str, Any]] = UNSET
        if self.metadata and not isinstance(self.metadata, Unset) and not isinstance(self.metadata, dict):
            metadata = self.metadata.to_dict()
        elif self.metadata and isinstance(self.metadata, dict):
            metadata = self.metadata

        spec: Union[Unset, dict[str, Any]] = UNSET
        if self.spec and not isinstance(self.spec, Unset) and not isinstance(self.spec, dict):
            spec = self.spec.to_dict()
        elif self.spec and isinstance(self.spec, dict):
            spec = self.spec

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if spec is not UNSET:
            field_dict["spec"] = spec

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.custom_domain_metadata import CustomDomainMetadata
        from ..models.custom_domain_spec import CustomDomainSpec

        if not src_dict:
            return None
        d = src_dict.copy()
        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, CustomDomainMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = CustomDomainMetadata.from_dict(_metadata)

        _spec = d.pop("spec", UNSET)
        spec: Union[Unset, CustomDomainSpec]
        if isinstance(_spec, Unset):
            spec = UNSET
        else:
            spec = CustomDomainSpec.from_dict(_spec)

        custom_domain = cls(
            metadata=metadata,
            spec=spec,
        )

        custom_domain.additional_properties = d
        return custom_domain

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="JobExecutionTaskMetadata")


@_attrs_define
class JobExecutionTaskMetadata:
    """Job execution task metadata

    Attributes:
        completed_at (Union[Unset, str]): Completion timestamp
        created_at (Union[Unset, str]): Creation timestamp
        name (Union[Unset, str]): Task name
        scheduled_at (Union[Unset, str]): Scheduled timestamp
        started_at (Union[Unset, str]): Start timestamp
        updated_at (Union[Unset, str]): Last update timestamp
    """

    completed_at: Union[Unset, str] = UNSET
    created_at: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    scheduled_at: Union[Unset, str] = UNSET
    started_at: Union[Unset, str] = UNSET
    updated_at: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        completed_at = self.completed_at

        created_at = self.created_at

        name = self.name

        scheduled_at = self.scheduled_at

        started_at = self.started_at

        updated_at = self.updated_at

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if completed_at is not UNSET:
            field_dict["completedAt"] = completed_at
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if name is not UNSET:
            field_dict["name"] = name
        if scheduled_at is not UNSET:
            field_dict["scheduledAt"] = scheduled_at
        if started_at is not UNSET:
            field_dict["startedAt"] = started_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        completed_at = d.pop("completedAt", UNSET)

        created_at = d.pop("createdAt", UNSET)

        name = d.pop("name", UNSET)

        scheduled_at = d.pop("scheduledAt", UNSET)

        started_at = d.pop("startedAt", UNSET)

        updated_at = d.pop("updatedAt", UNSET)

        job_execution_task_metadata = cls(
            completed_at=completed_at,
            created_at=created_at,
            name=name,
            scheduled_at=scheduled_at,
            started_at=started_at,
            updated_at=updated_at,
        )

        job_execution_task_metadata.additional_properties = d
        return job_execution_task_metadata

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
